<?php
// vim: set ts=4 sw=4 sts=4 et:

/**
 * X-Cart
 *
 *
 * @category  X-Cart 5
 * @author    Z-Payment
 * @copyright Copyright (c) 2011-2014 Z-Payment
 * @link      https://z-payment.com/
 */


namespace XLite\Module\Zpayment\Zpayment\Model\Payment\Processor;

use XLite\Core\Config;
use XLite\Core\Converter;
use XLite\Core\Request;
use XLite\Core\Translation;
use XLite\Core\XML;
use XLite\Model\Payment\Method;
use XLite\Model\Payment\Transaction;
use XLite\Model\Order;
use XLite\Model\Profile;
use XLite\Core\Database;
/**
 * Z-Payment payment processor
 */
class Zpayment extends \XLite\Model\Payment\Base\WebBased
{
    /**
     * Form URL
     */
    const FORM_URL = 'https://z-payment.com/merchant.php';


    /**
     * Allowed currencies codes
     *
     * @var   array
     */
    protected $allowedCurrencies = array('RUB', 'USD');

    /**
     * Get settings widget or template
     *
     * @return string Widget class name or template path
     */
    public function getSettingsWidget()
    {
        return 'modules/Zpayment/Zpayment/config.tpl';
    }

    /**
     * Check - payment method is configured or not
     *
     * @param Method $method Payment method
     *
     * @return boolean
     */
    public function isConfigured(Method $method)
    {
        $result = parent::isConfigured($method)
            && $method->getSetting('shop_id')
            && $method->getSetting('merchant_key')
			&& $method->getSetting('initialization_password');


        return $result;
    }

    /**
     * Payment method has settings into Module settings section
     *
     * @return boolean
     */
    public function hasModuleSettings()
    {
        return false;
    }
	
	    /**
     * Detect transaction
     *
     * @return \XLite\Model\Payment\Transaction
     */
    public function getReturnOwnerTransaction()
    {
        $request = Request::getInstance();

        return Database::getRepo('XLite\Model\Payment\Transaction')
            ->findOneBy(array('transaction_id' => $request->txnId));
    }
	
	

    /**
     * Process return
     *
     * @param Transaction $transaction Return-owner transaction
     *
     * @return void
     */
    public function processReturn(Transaction $transaction)
    {
        parent::processReturn($transaction);

        $request = Request::getInstance();
		$this->transaction->setStatus($transaction::STATUS_SUCCESS);


    }

    /**
     * Process callback
     *
     * @param Transaction $transaction Callback-owner transaction
     *
     * @return void
     */
    public function processCallback(Transaction $transaction)
    {
        parent::processCallback($transaction);
        $request = Request::getInstance();

        if (isset($request->LMI_PAYMENT_NO))
        {
            if(isset($request->LMI_PREREQUEST) && $request->LMI_PREREQUEST==1) { die('YES'); }

            $LMI_PAYMENT_NO = $request->LMI_PAYMENT_NO;
            $LMI_PAYEE_PURSE = $request->LMI_PAYEE_PURSE;
            $LMI_PAYMENT_AMOUNT = $request->LMI_PAYMENT_AMOUNT;
            $LMI_SECRET_KEY = $request->LMI_SECRET_KEY;
            $LMI_MODE = $request->LMI_MODE;
            $LMI_SYS_INVS_NO = $request->LMI_SYS_INVS_NO;
            $LMI_SYS_TRANS_NO = $request->LMI_SYS_TRANS_NO;
            $LMI_SYS_TRANS_DATE = $request->LMI_SYS_TRANS_DATE;
            $LMI_PAYER_PURSE = $request->LMI_PAYER_PURSE;
            $LMI_PAYER_WM = $request->LMI_PAYER_WM;
            $LMI_HASH = $request->LMI_HASH;

            $shop_id = $this->getSetting('shop_id');
            $shopSecret = $this->getSetting('merchant_key');
            $err=FALSE;
			$amount = number_format($this->transaction->getValue(), 2, '.', '');

            if ($shop_id!=$LMI_PAYEE_PURSE) {
                $err=TRUE;
                $errText="Shop Id mismatch";
            }
            if ($amount != $LMI_PAYMENT_AMOUNT) {
                $err=TRUE;
                $errText="Amount mismatch";
            }
			
            if(isset($LMI_SECRET_KEY)) {
                if($LMI_SECRET_KEY!=$shopSecret) {
                    $err = TRUE;
                    $errText='secret key mismatch';
                }
            } else {
                $CalcHash = md5($LMI_PAYEE_PURSE.$LMI_PAYMENT_AMOUNT.$LMI_PAYMENT_NO.$LMI_MODE.$LMI_SYS_INVS_NO.$LMI_SYS_TRANS_NO.$LMI_SYS_TRANS_DATE.$shopSecret.$LMI_PAYER_PURSE.$LMI_PAYER_WM);
                    if($LMI_HASH != strtoupper($CalcHash)) {
                        $err = TRUE;
                        $errText='hash mismatch';
                    }
            }
            if (!$err) {
                $this->transaction->setStatus($transaction::STATUS_SUCCESS);
				//$this->getOrder()->setPaymentStatus('P');
				Database::getRepo('XLite\Model\Payment\Transaction')->findOneBy(array('transaction_id' => $request->txnId ))->setStatus($transaction::STATUS_SUCCESS);
				Database::getRepo('XLite\Model\Order')->findOneBy(array('orderNumber' => $LMI_PAYMENT_NO ))->setPaymentStatus('P');
				Database::getEM()->flush();
                die('YES');
            } else {
                $this->transaction->setStatus($transaction::STATUS_FAILED);
				Database::getRepo('XLite\Model\Order')->findOneBy(array('orderNumber' => $LMI_PAYMENT_NO ))->setPaymentStatus('D');
				Database::getEM()->flush();
                die("FAIL: $errText");
            }
        } else die("FAIL");


    }

    /**
     * Get payment method admin zone icon URL
     *
     * @param Method $method Payment method
     *
     * @return string
     */
    public function getAdminIconURL(Method $method)
    {
        return true;
    }

    /**
     * Get payment form URL
     *
     * @return string
     */
    protected function getFormURL()
    {
        return static::FORM_URL;
    }

    /**
     * Get redirect form fields list
     *
     * @return array
     */
    protected function getFormFields()
    {
        $transactionId = $this->transaction->getTransactionId();
        $orderNum = $this->getOrder()->getOrderNumber();

        
        $amount = number_format($this->transaction->getValue(), 2, '.', '');
        $paymentDesc = 'Order X '. $orderNum;
	    $zpSign=md5($this->getSetting('shop_id').$orderNum.$this->transaction->getValue().$this->getSetting('initialization_password'));

        return array(
            'LMI_PAYEE_PURSE'       => $this->getSetting('shop_id'),
            'LMI_PAYMENT_AMOUNT'    => $amount,
            'LMI_PAYMENT_NO'        => $orderNum,
            'LMI_PAYMENT_DESC'      => $paymentDesc,
	        'ZP_SIGN'               => $zpSign,
            'txnId'                 => $transactionId,
        );
    }

    /**
     * Get allowed currencies
     *
     * @param Method $method Payment method
     *
     * @return array
     */
    protected function getAllowedCurrencies(Method $method)
    {
        return $this->allowedCurrencies;
    }

}
